{-# LANGUAGE TypeApplications #-}


-- | Copyright  : Will Thompson and Iñaki García Etxebarria
-- License    : LGPL-2.1
-- Maintainer : Iñaki García Etxebarria
-- 
-- A type corresponding to the appropriate struct type for the @/stat()/@
-- system call, depending on the platform and\/or compiler being used.
-- 
-- See 'GI.GLib.Functions.stat' for more information.

#if (MIN_VERSION_haskell_gi_overloading(1,0,0) && !defined(__HADDOCK_VERSION__))
#define ENABLE_OVERLOADING
#endif

module GI.GLib.Structs.StatBuf
    ( 

-- * Exported types
    StatBuf(..)                             ,


 -- * Methods

#if defined(ENABLE_OVERLOADING)
    ResolveStatBufMethod                    ,
#endif



    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.BasicTypes as B.Types
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GArray as B.GArray
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GHashTable as B.GHT
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.GI.Base.Signals as B.Signals
import qualified Control.Monad.IO.Class as MIO
import qualified Data.Coerce as Coerce
import qualified Data.Text as T
import qualified Data.Kind as DK
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL
import qualified GHC.Records as R
import qualified Data.Word as DW
import qualified Data.Int as DI
import qualified System.Posix.Types as SPT
import qualified Foreign.C.Types as FCT

-- Workaround for https://gitlab.haskell.org/ghc/ghc/-/issues/23392
#if MIN_VERSION_base(4,18,0)

#else

#endif

-- | Memory-managed wrapper type.
newtype StatBuf = StatBuf (SP.ManagedPtr StatBuf)
    deriving (Eq)

instance SP.ManagedPtrNewtype StatBuf where
    toManagedPtr (StatBuf p) = p

-- XXX Wrapping a foreign struct/union with no known destructor or size, leak?
instance BoxedPtr StatBuf where
    boxedPtrCopy = return
    boxedPtrFree = \_x -> return ()


#if defined(ENABLE_OVERLOADING)
instance O.HasAttributeList StatBuf
type instance O.AttributeList StatBuf = StatBufAttributeList
type StatBufAttributeList = ('[ ] :: [(Symbol, DK.Type)])
#endif

#if defined(ENABLE_OVERLOADING)
type family ResolveStatBufMethod (t :: Symbol) (o :: DK.Type) :: DK.Type where
    ResolveStatBufMethod l o = O.MethodResolutionFailed l o

instance (info ~ ResolveStatBufMethod t StatBuf, O.OverloadedMethod info StatBuf p) => OL.IsLabel t (StatBuf -> p) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.overloadedMethod @info
#else
    fromLabel _ = O.overloadedMethod @info
#endif

#if MIN_VERSION_base(4,13,0)
instance (info ~ ResolveStatBufMethod t StatBuf, O.OverloadedMethod info StatBuf p, R.HasField t StatBuf p) => R.HasField t StatBuf p where
    getField = O.overloadedMethod @info

#endif

instance (info ~ ResolveStatBufMethod t StatBuf, O.OverloadedMethodInfo info StatBuf) => OL.IsLabel t (O.MethodProxy info StatBuf) where
#if MIN_VERSION_base(4,10,0)
    fromLabel = O.MethodProxy
#else
    fromLabel _ = O.MethodProxy
#endif

#endif


